/** Rill v0.0.595 https://www.npmjs.com/package/rill License: MIT*/
"use strict";
var Request, Response, Rill, co, compose, expect, http, respond;

co = require("co");

compose = require("@rill/compose");

expect = require("./util/expect");

respond = require("./util/respond");

Request = require("./request");

Response = require("./response");

http = (require("./util/browser"));

Rill = (function() {

  /*
  	 * Creates an isomorphic Rill application that can listen for browser and server page requests.
  	 *
  	 * @constructor
   */
  function Rill() {
    if (!(this instanceof Rill)) {
      return new Rill();
    }
    this.middleware = [];
  }


  /*
  	 * Starts the server or browser hijacker.
   */

  Rill.prototype.listen = function() {
    var fn, ref;
    fn = compose(this.middleware);
    return (ref = http.createServer(function(req, res) {
      if (res == null) {
        res = {};
      }
      res.statusCode = 404;
      return fn.call({}, req = new Request(req), res = new Response(res)).then(function() {
        respond(req, res);
        return {
          redirected: Boolean(req.redirectURL)
        };
      })["catch"](function(err) {
        throw setTimeout((function() {
          throw err;
        }), 0);
      });
    })).listen.apply(ref, arguments);
  };


  /*
  	 * Simple syntactic sugar for functions that wish to modify the current application instance.
  	 *
  	 * @param {Function} transformer
   */

  Rill.prototype.setup = function(transformer) {
    transformer(this);
    return this;
  };


  /*
  	 * Push another rill application or middleware onto the current stack of middlewares.
  	 *
  	 * @params {Rill|Function|null} fn
   */

  Rill.prototype.use = function(fn) {
    if (!fn) {
      return this;
    }
    if (fn.middleware != null) {
      fn = compose(fn.middleware);
    }
    expect(typeof fn === "function", "Can only use functions and other Rill applications.");
    this.middleware.push(fn);
    return this;
  };


  /*
  	 * Expose internal utilites.
   */

  Rill.prototype.util = {
    compose: compose,
    expect: expect
  };

  return Rill;

})();

module.exports = Rill;
