co        = require("co")
compose   = require("@rill/compose")
expect    = require("./util/expect")
respond   = require("./util/respond")
Request   = require("./request")
Response  = require("./response")
http      = (
	if process.browser then require("./util/browser")
	else require("http")
)

class Rill
	###
	# Creates an isomorphic Rill application that can listen for browser and server page requests.
	#
	# @constructor
	###
	constructor: ->
		return new Rill() unless this instanceof Rill
		@middleware = []

	###
	# Starts the server or browser hijacker.
	###
	listen: ->
		fn = compose(@middleware)
		http.createServer((req, res = {})->
			res.statusCode = 404
			fn.call({}, req = new Request(req), res = new Response(res))
				.then(->
					respond(req, res)
					redirected: Boolean(req.redirectURL)
				)
				.catch((err)-> throw setTimeout((-> throw err), 0))
		).listen(arguments...)

	###
	# Simple syntactic sugar for functions that wish to modify the current application instance.
	#
	# @param {Function} transformer
	###
	setup: (transformer)->
		transformer(@)
		return this

	###
	# Push another rill application or middleware onto the current stack of middlewares.
	#
	# @params {Rill|Function|null} fn
	###
	use: (fn)->
		# Make empty arguments a noop.
		return this unless fn
		# Allow other apps.
		fn = compose(fn.middleware) if fn.middleware?

		expect(typeof fn is "function", "Can only use functions and other Rill applications.")
		@middleware.push(fn)
		return this

	###
	# Expose internal utilites.
	###
	util: { compose, expect }

module.exports = Rill